function results = testconvex(p,q)
%%%%Linear Program for Single Product Oligopoly
%p is a tx1 vector of prices
%qin is a txk vector of quantities for the k firms

p=p';           %Program only works when these are transposed
q=q';
[N T]=size(q);  %Number of firms, number of periods
ceq=0;          %number of equality constraints (common ratio property)
c=0;            %number of inequality constraints (

b=zeros(1,1);   %Make the vectors of RHS constants empty to begin with
beq=zeros(1,1);

%The vector of unknowns is (where d is deltabar, e is epsilon)
%[d_11,...,d_N1,...,d1T...,dNT,e11,...d_11,...,e_N1,...,e1T...,eNT]

%First set of constraints: equalities of the (P-delta)/Q terms
    %i.e. common ratio property
    %Constraints are:-d(i,t)/q(i,t)+d(j,t)/q(j,t)=-p(t)/q(i,t)+p(t)/q(j,t)
for t=1 : T                 
    for i=1:1
        for j=i+1:N
        ceq=ceq+1;
        Aeq(ceq,N*(t-1)+i)=-1/q(i,t);       %Coefficient on delta-bar^it
        Aeq(ceq,N*(t-1)+j)=1/q(j,t);        %Coefficient on delta-bar^jt
        beq(ceq)=-p(t)/q(i,t)+p(t)/q(j,t);  %RHS constant 
        end
    end
end
  
%Second set of constraints are the (P-delta)/Q>0 inequalities
%But quadprog only accepts "less than" constraints
%so this becomes d(i,t)<p(t)
  for t=1 : T 
    for i=1:N
        c=c+1;
        A(c,N*(t-1)+i)=1;      %Coefficient on delta-bar^it
        b(c)=p(t);             %The equality constraint within a single year 
    end
  end
  
%Third set of constraints are inequalities:
%i.e. delta-bar is increasing in Q.
%with delta-bar minus epsilon
%Also, the signs are reversed because quadprog accepts only "less than"
%constraints: -(q(i,t)-q(i,tt))(d(i,t)-d(i,tt))<0
for i=1:N
    for t=1 : T 
        for tt=t+1:T
        c=c+1;
        A(c,N*(t-1)+i)=-(q(i,t)-q(i,tt));       %Coefficient on deltabar^it
        A(c,N*(tt-1)+i)=q(i,t)-q(i,tt);         %Coefficient on deltabar^itt
        b(c)=0;                                 %The sum must be negative
        end
    end
end

%Define the objective function
H=[zeros(N*T)];                                  %x'*H*x should be 0: No errors in this test
f=zeros(N*T,1)  ;                                %f'*x should be 0
UB=Inf*ones(N*T,1)  ;                            %No upper bound on the parameters
LB=zeros(N*T,1)   ;                              %Deltabars must be positive

[x,L,exitflag] = quadprog(H,f,A,b,Aeq,beq,LB,UB) ;
results.exit=exitflag ;
results.reject=(exitflag~=1) ;

%x is the vector of deltabars and epsilons
%L is the test statistic
%exitflag tells me if it worked
%The program accepts Ax<b constraints, but our constraints are
%Ax>b, so enter it as -Ax<-b
%Equality constraints are fine as is

end

